@@ -90,7 +90,9 @@ bson_string_free (bson_string_t *string, /* IN */
 {
    char *ret = NULL;
 
-   BSON_ASSERT (string);
+   if (!string) {
+      return NULL;
+   }
 
    if (!free_segment) {
       ret = string->str;
@@ -712,7 +714,7 @@ bson_ascii_strtoll (const char *s, char **e, int base)
 
    c = *tok;
 
-   while (c >= -1 && c <= 255 && isspace (c)) {
+   while (bson_isspace (c)) {
       c = *++tok;
    }
 
@@ -811,3 +813,10 @@ bson_strcasecmp (const char *s1, const char *s2)
    return strcasecmp (s1, s2);
 #endif
 }
+
+
+bool
+bson_isspace (int c)
+{
+   return c >= -1 && c <= 255 && isspace (c);
+}
