@@ -18,10 +18,10 @@
 #define MONGOC_LOG_DOMAIN "topology-test"
 
 typedef struct {
-   int n_started;
-   int n_succeeded;
-   int n_failed;
-   int n_unknowns;
+   size_t n_started;
+   size_t n_succeeded;
+   size_t n_failed;
+   size_t n_unknowns;
    bson_mutex_t mutex;
 } checks_t;
 
@@ -39,9 +39,9 @@ checks_cleanup (checks_t *checks)
 }
 
 static bool
-checks_cmp (checks_t *checks, const char *metric, char cmp, int expected)
+checks_cmp (checks_t *checks, const char *metric, char cmp, size_t expected)
 {
-   int actual = 0;
+   size_t actual = 0;
 
    bson_mutex_lock (&checks->mutex);
    if (0 == strcmp (metric, "n_started")) {
@@ -181,7 +181,7 @@ test_topology_client_creation (void)
 
    /* ensure that we are sharing streams with the client */
    server_stream = mongoc_cluster_stream_for_reads (
-      &client_a->cluster, NULL, NULL, NULL, false, &error);
+      &client_a->cluster, NULL, NULL, NULL, &error);
 
    ASSERT_OR_PRINT (server_stream, error);
    node = mongoc_topology_scanner_get_node (client_a->topology->scanner,
@@ -370,7 +370,7 @@ _test_server_selection (bool try_once)
       client->topology, MONGOC_SS_READ, primary_pref, NULL, &error);
    request = mock_server_receives_any_hello (server);
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, secondary_response);
+   reply_to_request_simple (request, secondary_response);
    request_destroy (request);
 
    /* the selection timeout is 100 ms, and we can't rescan until a half second
@@ -403,7 +403,7 @@ _test_server_selection (bool try_once)
    BSON_ASSERT (request);
 
    /* the secondary is now primary, selection succeeds */
-   mock_server_replies_simple (request, primary_response);
+   reply_to_request_simple (request, primary_response);
    sd = future_get_mongoc_server_description_ptr (future);
    BSON_ASSERT (sd);
    BSON_ASSERT (!client->topology->stale);
@@ -456,7 +456,6 @@ _test_topology_invalidate_server (bool pooled)
 {
    mongoc_server_description_t *fake_sd;
    const mongoc_server_description_t *sd;
-   mongoc_uri_t *uri;
    mongoc_client_t *client;
    mongoc_client_pool_t *pool = NULL;
    bson_error_t error;
@@ -465,18 +464,18 @@ _test_topology_invalidate_server (bool pooled)
    uint32_t id;
    mongoc_server_stream_t *server_stream;
    checks_t checks;
-   int server_count;
-   mongoc_apm_callbacks_t *callbacks;
    mc_shared_tpld td;
    mc_tpld_modification tdmod;
 
    checks_init (&checks);
-   uri = test_framework_get_uri ();
+
+   mongoc_uri_t *const uri = test_framework_get_uri ();
    /* no auto heartbeat */
    mongoc_uri_set_option_as_int32 (uri, "heartbeatFrequencyMS", INT32_MAX);
    mongoc_uri_set_option_as_int32 (uri, "connectTimeoutMS", 3000);
-   server_count = test_framework_server_count ();
-   callbacks = heartbeat_callbacks ();
+
+   const size_t server_count = test_framework_server_count ();
+   mongoc_apm_callbacks_t *const callbacks = heartbeat_callbacks ();
 
    if (pooled) {
       pool = test_framework_client_pool_new_from_uri (uri, NULL);
@@ -496,7 +495,7 @@ _test_topology_invalidate_server (bool pooled)
 
    /* call explicitly */
    server_stream = mongoc_cluster_stream_for_reads (
-      &client->cluster, NULL, NULL, NULL, false, &error);
+      &client->cluster, NULL, NULL, NULL, &error);
    ASSERT_OR_PRINT (server_stream, error);
    sd = server_stream->sd;
    id = server_stream->sd->id;
@@ -603,7 +602,7 @@ test_invalid_cluster_node (void *ctx)
 
    /* load stream into cluster */
    server_stream = mongoc_cluster_stream_for_reads (
-      &client->cluster, NULL, NULL, NULL, false, &error);
+      &client->cluster, NULL, NULL, NULL, &error);
    ASSERT_OR_PRINT (server_stream, error);
    id = server_stream->sd->id;
    mongoc_server_stream_cleanup (server_stream);
@@ -676,7 +675,7 @@ test_max_wire_version_race_condition (void *ctx)
 
    /* load stream into cluster */
    server_stream = mongoc_cluster_stream_for_reads (
-      &client->cluster, NULL, NULL, NULL, false, &error);
+      &client->cluster, NULL, NULL, NULL, &error);
    ASSERT_OR_PRINT (server_stream, error);
    id = server_stream->sd->id;
    mongoc_server_stream_cleanup (server_stream);
@@ -724,7 +723,7 @@ test_cooldown_standalone (void)
       client->topology, MONGOC_SS_READ, primary_pref, NULL, &error);
    request = mock_server_receives_any_hello (server);
    BSON_ASSERT (request);
-   mock_server_hangs_up (request);
+   reply_to_request_with_hang_up (request);
    BSON_ASSERT (!future_get_mongoc_server_description_ptr (future));
    request_destroy (request);
    future_destroy (future);
@@ -765,13 +764,13 @@ test_cooldown_standalone (void)
       client->topology, MONGOC_SS_READ, primary_pref, NULL, &error);
    request = mock_server_receives_any_hello (server); /* not in cooldown now */
    BSON_ASSERT (request);
-   mock_server_replies_simple (request,
-                               tmp_str ("{'ok': 1,"
-                                        " 'isWritablePrimary': true,"
-                                        " 'minWireVersion': %d,"
-                                        " 'maxWireVersion': %d}",
-                                        WIRE_VERSION_MIN,
-                                        WIRE_VERSION_MAX));
+   reply_to_request_simple (request,
+                            tmp_str ("{'ok': 1,"
+                                     " 'isWritablePrimary': true,"
+                                     " 'minWireVersion': %d,"
+                                     " 'maxWireVersion': %d}",
+                                     WIRE_VERSION_MIN,
+                                     WIRE_VERSION_MAX));
    sd = future_get_mongoc_server_description_ptr (future);
    BSON_ASSERT (sd);
    request_destroy (request);
@@ -788,7 +787,6 @@ static void
 test_cooldown_rs (void)
 {
    mock_server_t *servers[2]; /* two secondaries, no primary */
-   char *uri_str;
    int i;
    mongoc_client_t *client;
    mongoc_read_prefs_t *primary_pref;
@@ -804,12 +802,25 @@ test_cooldown_rs (void)
       mock_server_run (servers[i]);
    }
 
-   uri_str = bson_strdup_printf ("mongodb://localhost:%hu/?replicaSet=rs"
-                                 "&serverSelectionTimeoutMS=100"
-                                 "&connectTimeoutMS=100",
-                                 mock_server_get_port (servers[0]));
+   {
+      char *uri_str =
+         bson_strdup_printf ("mongodb://localhost:%hu/?replicaSet=rs"
+                             "&serverSelectionTimeoutMS=100"
+                             "&connectTimeoutMS=100",
+                             mock_server_get_port (servers[0]));
+
+      mongoc_uri_t *const uri = mongoc_uri_new_with_error (uri_str, &error);
+      ASSERT_OR_PRINT (uri, error);
+
+      // Prevent retryable handshakes from interfering with mock server hangups.
+      mongoc_uri_set_option_as_bool (uri, MONGOC_URI_RETRYREADS, false);
+
+      client = test_framework_client_new_from_uri (uri, NULL);
+
+      bson_free (uri_str);
+      mongoc_uri_destroy (uri);
+   }
 
-   client = test_framework_client_new (uri_str, NULL);
    primary_pref = mongoc_read_prefs_new (MONGOC_READ_PRIMARY);
 
    secondary_response =
@@ -841,13 +852,13 @@ test_cooldown_rs (void)
 
    request = mock_server_receives_any_hello (servers[0]);
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, secondary_response);
+   reply_to_request_simple (request, secondary_response);
    request_destroy (request);
 
    /* server 0 told us about server 1. we check it immediately but it's down. */
    request = mock_server_receives_any_hello (servers[1]);
    BSON_ASSERT (request);
-   mock_server_hangs_up (request);
+   reply_to_request_with_hang_up (request);
    request_destroy (request);
 
    /* selection fails. */
@@ -862,7 +873,7 @@ test_cooldown_rs (void)
 
    request = mock_server_receives_any_hello (servers[0]);
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, secondary_response);
+   reply_to_request_simple (request, secondary_response);
    request_destroy (request);
 
    mock_server_set_request_timeout_msec (servers[1], 100);
@@ -881,7 +892,7 @@ test_cooldown_rs (void)
 
    request = mock_server_receives_any_hello (servers[1]);
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, primary_response);
+   reply_to_request_simple (request, primary_response);
    request_destroy (request);
 
    /* server 0 doesn't need to respond */
@@ -894,7 +905,6 @@ test_cooldown_rs (void)
    mongoc_client_destroy (client);
    bson_free (secondary_response);
    bson_free (primary_response);
-   bson_free (uri_str);
    mock_server_destroy (servers[0]);
    mock_server_destroy (servers[1]);
 }
@@ -929,7 +939,7 @@ test_cooldown_retry (void)
    /* first hello fails */
    request = mock_server_receives_any_hello (server);
    BSON_ASSERT (request);
-   mock_server_hangs_up (request);
+   reply_to_request_with_hang_up (request);
    request_destroy (request);
 
    /* after cooldown passes, driver sends another hello */
@@ -941,13 +951,13 @@ test_cooldown_retry (void)
    ASSERT_CMPINT64 (duration, >, (int64_t) 5 * 1000 * 1000);
    ASSERT_CMPINT64 (duration, <, (int64_t) 10 * 1000 * 1000);
 
-   mock_server_replies_simple (request,
-                               tmp_str ("{'ok': 1,"
-                                        " 'isWritablePrimary': true,"
-                                        " 'minWireVersion': %d,"
-                                        " 'maxWireVersion': %d}",
-                                        WIRE_VERSION_MIN,
-                                        WIRE_VERSION_MAX));
+   reply_to_request_simple (request,
+                            tmp_str ("{'ok': 1,"
+                                     " 'isWritablePrimary': true,"
+                                     " 'minWireVersion': %d,"
+                                     " 'maxWireVersion': %d}",
+                                     WIRE_VERSION_MIN,
+                                     WIRE_VERSION_MAX));
    sd = future_get_mongoc_server_description_ptr (future);
    ASSERT_OR_PRINT (sd, error);
    request_destroy (request);
@@ -1029,9 +1039,7 @@ _test_select_succeed (bool try_once)
 
    duration_usec = bson_get_monotonic_time () - start;
 
-   if (!test_suite_valgrind ()) {
-      ASSERT_ALMOST_EQUAL (duration_usec / 1000, connect_timeout_ms);
-   }
+   ASSERT_ALMOST_EQUAL (duration_usec / 1000, connect_timeout_ms);
 
    mongoc_client_destroy (client);
    mongoc_uri_destroy (uri);
@@ -1118,7 +1126,7 @@ auto_ping (request_t *request, void *data)
       return false;
    }
 
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
 
    return true;
 }
@@ -1281,7 +1289,7 @@ test_rtt (void *ctx)
 
    request = mock_server_receives_any_hello (server);
    _mongoc_usleep (1000 * 1000); /* one second */
-   mock_server_replies (
+   reply_to_request (
       request,
       MONGOC_REPLY_NONE,
       0,
@@ -1293,7 +1301,7 @@ test_rtt (void *ctx)
    request_destroy (request);
    request = mock_server_receives_msg (
       server, MONGOC_MSG_NONE, tmp_bson ("{'$db': 'db', 'ping': 1}"));
-   mock_server_replies (
+   reply_to_request (
       request,
       MONGOC_REPLY_NONE,
       0,
@@ -1359,18 +1367,26 @@ test_add_and_scan_failure (void)
 
    request = mock_server_receives_msg (
       server, MONGOC_MSG_NONE, tmp_bson ("{'$db': 'db', 'ping': 1}"));
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
 
-   sd = mongoc_topology_description_server_by_id_const (
-      mc_tpld_unsafe_get_const (client->topology), 1, NULL);
-   ASSERT (sd);
-   ASSERT_CMPSTR (mongoc_server_description_type (sd), "RSPrimary");
+   {
+      mc_shared_tpld shared_tpld = mc_tpld_take_ref (client->topology);
+      sd = mongoc_topology_description_server_by_id_const (
+         shared_tpld.ptr, 1, NULL);
+      ASSERT (sd);
+      ASSERT_CMPSTR (mongoc_server_description_type (sd), "RSPrimary");
+      mc_tpld_drop_ref (&shared_tpld);
+   }
 
-   sd = mongoc_topology_description_server_by_id_const (
-      mc_tpld_unsafe_get_const (client->topology), 2, NULL);
-   ASSERT (sd);
-   ASSERT_CMPSTR (mongoc_server_description_type (sd), "Unknown");
+   {
+      mc_shared_tpld shared_tpld = mc_tpld_take_ref (client->topology);
+      sd = mongoc_topology_description_server_by_id_const (
+         mc_tpld_unsafe_get_const (client->topology), 2, NULL);
+      ASSERT (sd);
+      ASSERT_CMPSTR (mongoc_server_description_type (sd), "Unknown");
+      mc_tpld_drop_ref (&shared_tpld);
+   }
 
    future_destroy (future);
    mongoc_client_pool_push (pool, client);
@@ -1396,7 +1412,7 @@ receives_command (mock_server_t *server, future_t *future)
 
    request = mock_server_receives_msg (
       server, MONGOC_MSG_NONE, tmp_bson ("{'$db': 'admin', 'foo': 1}"));
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
 }
@@ -1408,6 +1424,8 @@ has_known_server (mongoc_client_t *client)
    mongoc_server_description_t *sd;
    bool r;
 
+   ASSERT (client);
+
    /* in this test we know the server id is always 1 */
    sd = mongoc_client_get_server_description (client, 1);
    r = (sd->type != MONGOC_SERVER_UNKNOWN);
@@ -1417,7 +1435,7 @@ has_known_server (mongoc_client_t *client)
 
 
 static void
-_test_hello_retry_single (bool hangup, int n_failures)
+_test_hello_retry_single (bool hangup, size_t n_failures)
 {
    checks_t checks;
    mongoc_apm_callbacks_t *callbacks;
@@ -1457,7 +1475,7 @@ _test_hello_retry_single (bool hangup, int n_failures)
    /* start a {foo: 1} command, handshake normally */
    future = future_command (client, &error);
    request = mock_server_receives_any_hello (server);
-   mock_server_replies_simple (request, hello);
+   reply_to_request_simple (request, hello);
    request_destroy (request);
    receives_command (server, future);
 
@@ -1469,7 +1487,7 @@ _test_hello_retry_single (bool hangup, int n_failures)
    request = mock_server_receives_any_hello (server);
    t = bson_get_monotonic_time ();
    if (hangup) {
-      mock_server_hangs_up (request);
+      reply_to_request_with_hang_up (request);
    }
 
    request_destroy (request);
@@ -1478,26 +1496,26 @@ _test_hello_retry_single (bool hangup, int n_failures)
    request = mock_server_receives_any_hello (server);
    ASSERT_CMPINT64 (bson_get_monotonic_time () - t, <, (int64_t) 250 * 1000);
 
-   if (n_failures == 2) {
+   if (n_failures == 2u) {
       if (hangup) {
-         mock_server_hangs_up (request);
+         reply_to_request_with_hang_up (request);
       }
 
       BSON_ASSERT (!future_get_bool (future));
       future_destroy (future);
    } else {
-      mock_server_replies_simple (request, hello);
+      reply_to_request_simple (request, hello);
       /* the {foo: 1} command finishes */
       receives_command (server, future);
    }
 
    request_destroy (request);
 
-   ASSERT_CMPINT (checks.n_started, ==, 3);
-   WAIT_UNTIL (checks.n_succeeded == 3 - n_failures);
+   ASSERT_CMPSIZE_T (checks.n_started, ==, 3u);
+   WAIT_UNTIL (checks.n_succeeded == 3u - n_failures);
    WAIT_UNTIL (checks.n_failed == n_failures);
 
-   if (n_failures == 2) {
+   if (n_failures == 2u) {
       BSON_ASSERT (!has_known_server (client));
    } else {
       BSON_ASSERT (has_known_server (client));
@@ -1513,7 +1531,7 @@ _test_hello_retry_single (bool hangup, int n_failures)
 
 
 static void
-_test_hello_retry_pooled (bool hangup, int n_failures)
+_test_hello_retry_pooled (bool hangup, size_t n_failures)
 {
    checks_t checks;
    mongoc_apm_callbacks_t *callbacks;
@@ -1554,7 +1572,7 @@ _test_hello_retry_pooled (bool hangup, int n_failures)
 
    /* As soon as the client is popped, background monitoring starts. */
    request = mock_server_receives_legacy_hello (server, NULL);
-   mock_server_replies_simple (request, hello);
+   reply_to_request_simple (request, hello);
    request_destroy (request);
 
    /* start a {foo: 1} command, handshake normally */
@@ -1562,7 +1580,7 @@ _test_hello_retry_pooled (bool hangup, int n_failures)
 
    /* Another hello to handshake the connection */
    request = mock_server_receives_legacy_hello (server, NULL);
-   mock_server_replies_simple (request, hello);
+   reply_to_request_simple (request, hello);
    request_destroy (request);
 
    /* the {foo: 1} command finishes */
@@ -1572,7 +1590,7 @@ _test_hello_retry_pooled (bool hangup, int n_failures)
    request = mock_server_receives_legacy_hello (server, NULL);
    t = bson_get_monotonic_time ();
    if (hangup) {
-      mock_server_hangs_up (request);
+      reply_to_request_with_hang_up (request);
    }
 
    request_destroy (request);
@@ -1588,20 +1606,20 @@ _test_hello_retry_pooled (bool hangup, int n_failures)
     * check without waiting".
     */
    BSON_ASSERT (!has_known_server (client));
-   if (n_failures == 2) {
+   if (n_failures == 2u) {
       if (hangup) {
-         mock_server_hangs_up (request);
+         reply_to_request_with_hang_up (request);
       }
    } else {
-      mock_server_replies_simple (request, hello);
+      reply_to_request_simple (request, hello);
       WAIT_UNTIL (has_known_server (client));
    }
 
    request_destroy (request);
 
-   WAIT_UNTIL (checks_cmp (&checks, "n_succeeded", '=', 3 - n_failures));
+   WAIT_UNTIL (checks_cmp (&checks, "n_succeeded", '=', 3u - n_failures));
    WAIT_UNTIL (checks_cmp (&checks, "n_failed", '=', n_failures));
-   BSON_ASSERT (checks_cmp (&checks, "n_started", '=', 3));
+   BSON_ASSERT (checks_cmp (&checks, "n_started", '=', 3u));
 
    mongoc_client_pool_push (pool, client);
    mongoc_client_pool_destroy (pool);
@@ -1616,55 +1634,55 @@ _test_hello_retry_pooled (bool hangup, int n_failures)
 static void
 test_hello_retry_single_hangup (void)
 {
-   _test_hello_retry_single (true, 1);
+   _test_hello_retry_single (true, 1u);
 }
 
 
 static void
 test_hello_retry_single_timeout (void)
 {
-   _test_hello_retry_single (false, 1);
+   _test_hello_retry_single (false, 1u);
 }
 
 static void
 test_hello_retry_single_hangup_fail (void)
 {
-   _test_hello_retry_single (true, 2);
+   _test_hello_retry_single (true, 2u);
 }
 
 
 static void
 test_hello_retry_single_timeout_fail (void)
 {
-   _test_hello_retry_single (false, 2);
+   _test_hello_retry_single (false, 2u);
 }
 
 
 static void
 test_hello_retry_pooled_hangup (void)
 {
-   _test_hello_retry_pooled (true, 1);
+   _test_hello_retry_pooled (true, 1u);
 }
 
 
 static void
 test_hello_retry_pooled_timeout (void)
 {
-   _test_hello_retry_pooled (false, 1);
+   _test_hello_retry_pooled (false, 1u);
 }
 
 
 static void
 test_hello_retry_pooled_hangup_fail (void)
 {
-   _test_hello_retry_pooled (true, 2);
+   _test_hello_retry_pooled (true, 2u);
 }
 
 
 static void
 test_hello_retry_pooled_timeout_fail (void)
 {
-   _test_hello_retry_pooled (false, 2);
+   _test_hello_retry_pooled (false, 2u);
 }
 
 
@@ -1987,7 +2005,7 @@ _test_request_scan_on_error (bool pooled,
    /* Capture logs to swallow warnings about endSessions */
    capture_logs (true);
 
-   mock_server_replies_simple (request, err_response);
+   reply_to_request_simple (request, err_response);
    request_destroy (request);
    /* don't check the return value of future. write concern errors are still
     * considered successful results. */
@@ -2010,8 +2028,8 @@ _test_request_scan_on_error (bool pooled,
          /* after the 'ping' command and returning, the server should
           * have been marked as unknown. */
          BSON_ASSERT (sd->type == MONGOC_SERVER_UNKNOWN);
-         ASSERT_CMPINT (sd->last_update_time_usec, >=, ping_started_usec);
-         ASSERT_CMPINT (
+         ASSERT_CMPINT64 (sd->last_update_time_usec, >=, ping_started_usec);
+         ASSERT_CMPINT64 (
             sd->last_update_time_usec, <=, bson_get_monotonic_time ());
          /* check that the error on the server description matches the error
           * message in the response. */
@@ -2046,7 +2064,7 @@ _test_request_scan_on_error (bool pooled,
          request = mock_server_receives_msg (
             secondary, MONGOC_QUERY_NONE, tmp_bson ("{'ping': 1}"));
       }
-      mock_server_replies_simple (request, "{'ok': 1}");
+      reply_to_request_simple (request, "{'ok': 1}");
       request_destroy (request);
       BSON_ASSERT (future_get_bool (future));
       future_destroy (future);
@@ -2293,7 +2311,7 @@ test_slow_server_pooled (void)
     * blocked for connectTimeoutMS. */
    mock_server_auto_hello (secondary, hello_secondary);
    /* Respond to the first hello. */
-   mock_server_replies_simple (request, hello_secondary);
+   reply_to_request_simple (request, hello_secondary);
    request_destroy (request);
 
    /* Now a command to the secondary succeeds. */
@@ -2366,7 +2384,7 @@ _test_hello_versioned_api (bool pooled)
    BSON_ASSERT (bson_has_field (request_get_doc (request, 0), "apiVersion"));
    BSON_ASSERT (bson_has_field (request_get_doc (request, 0), "helloOk"));
 
-   mock_server_replies_simple (request, hello_reply);
+   reply_to_request_simple (request, hello_reply);
 
    request_destroy (request);
 
@@ -2374,7 +2392,7 @@ _test_hello_versioned_api (bool pooled)
       request = mock_server_receives_msg (
          server, MONGOC_MSG_NONE, tmp_bson ("{'$db': 'admin', 'ping': 1}"));
 
-      mock_server_replies_ok_and_destroys (request);
+      reply_to_request_with_ok_and_destroy (request);
       BSON_ASSERT (future_get_bool (future));
       future_destroy (future);
    }
@@ -2461,14 +2479,14 @@ _test_hello_ok (bool pooled)
       server, NULL, "{'" HANDSHAKE_CMD_LEGACY_HELLO "': 1, 'helloOk': true}");
 
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, hello);
+   reply_to_request_simple (request, hello);
    request_destroy (request);
 
    /* For non-pooled clients, handle the ping */
    if (!pooled) {
       request = mock_server_receives_msg (
          server, MONGOC_MSG_NONE, tmp_bson ("{'$db': 'admin', 'ping': 1}"));
-      mock_server_replies_ok_and_destroys (request);
+      reply_to_request_with_ok_and_destroy (request);
       BSON_ASSERT (future_get_bool (future));
       future_destroy (future);
 
@@ -2482,7 +2500,7 @@ _test_hello_ok (bool pooled)
    /* Hang up to ensure that the next check runs legacy hello again */
    request = mock_server_receives_any_hello_with_match (server, "{}", "{}");
    BSON_ASSERT (request);
-   mock_server_hangs_up (request);
+   reply_to_request_with_hang_up (request);
    request_destroy (request);
 
    /* The previous failure will trigger another handshake using legacy hello */
@@ -2492,14 +2510,14 @@ _test_hello_ok (bool pooled)
       "{'" HANDSHAKE_CMD_LEGACY_HELLO "': 1, 'helloOk': true}");
 
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, hello_not_ok);
+   reply_to_request_simple (request, hello_not_ok);
    request_destroy (request);
 
    /* Once again, handle the ping */
    if (!pooled) {
       request = mock_server_receives_msg (
          server, MONGOC_MSG_NONE, tmp_bson ("{'$db': 'admin', 'ping': 1}"));
-      mock_server_replies_ok_and_destroys (request);
+      reply_to_request_with_ok_and_destroy (request);
       BSON_ASSERT (future_get_bool (future));
       future_destroy (future);
 
@@ -2518,14 +2536,14 @@ _test_hello_ok (bool pooled)
       "{'" HANDSHAKE_CMD_LEGACY_HELLO "': 1, 'helloOk': true}");
 
    BSON_ASSERT (request);
-   mock_server_replies_simple (request, hello_not_ok);
+   reply_to_request_simple (request, hello_not_ok);
    request_destroy (request);
 
    /* Once again, handle the ping */
    if (!pooled) {
       request = mock_server_receives_msg (
          server, MONGOC_MSG_NONE, tmp_bson ("{'$db': 'admin', 'ping': 1}"));
-      mock_server_replies_ok_and_destroys (request);
+      reply_to_request_with_ok_and_destroy (request);
       BSON_ASSERT (future_get_bool (future));
       future_destroy (future);
    }
@@ -2588,15 +2606,13 @@ test_topology_install (TestSuite *suite)
                       test_topology_invalidate_server_single,
                       NULL,
                       NULL,
-                      test_framework_skip_if_slow_or_live,
-                      test_framework_skip_if_valgrind);
+                      test_framework_skip_if_slow_or_live);
    TestSuite_AddFull (suite,
                       "/Topology/invalidate_server/pooled",
                       test_topology_invalidate_server_pooled,
                       NULL,
                       NULL,
-                      test_framework_skip_if_slow_or_live,
-                      test_framework_skip_if_valgrind);
+                      test_framework_skip_if_slow_or_live);
    TestSuite_AddFull (suite,
                       "/Topology/invalid_cluster_node",
                       test_invalid_cluster_node,
